import { createEffect as effectorCreateEffect, createEvent } from 'effector';
import { patchInstance } from './instance';
import { patchRunner } from './runner';
import { TAKE_EVERY } from './strategy';
export const createReEffectFactory = (createEffect = effectorCreateEffect) => (nameOrConfig, maybeConfig) => {
    const instance = createEffect(nameOrConfig, maybeConfig);
    const cancelled = createEvent({ named: 'cancelled' });
    const cancel = createEvent({ named: 'cancel' });
    const config = maybeConfig
        ? maybeConfig
        : nameOrConfig && typeof nameOrConfig === 'object'
            ? nameOrConfig
            : {};
    const running = [];
    const scope = {
        strategy: config.strategy || TAKE_EVERY,
        feedback: config.feedback || false,
        limit: config.limit || Infinity,
        timeout: config.timeout,
        cancelled,
        cancel,
        running,
        inFlight: instance.inFlight,
        push: (promise) => running.push(promise),
        unpush: (promise) => {
            if (promise) {
                running.splice(running.indexOf(promise), 1);
            }
            return running.length;
        },
        cancelAll: (strategy) => running.map(promise => promise.cancel(strategy)),
    };
    patchRunner(instance.graphite.scope.runner, scope);
    patchInstance(instance, scope);
    return instance;
};
