import { createNode, launch, step } from 'effector';
import { CancelledError, LimitExceededError } from './error';
import { QUEUE, RACE, TAKE_FIRST, TAKE_LAST } from './strategy';
import { cancellable } from './promise';
import { assign } from './tools';
export const patchRunner = (runner, scope) => {
    assign(runner.scope, scope);
    runner.meta.onCopy.push('cancelled', 'cancel');
    runner.seq = seq();
    scope.cancel.watch(() => scope.cancelAll());
};
const seq = () => [
    step.run({
        fn({ params, args, req }, { getHandler, finally: anyway, strategy, feedback, limit, timeout, cancelled, running, inFlight, push, unpush, cancelAll, }) {
            strategy = (args && args.strategy) || strategy;
            timeout = (args && args.timeout) || timeout;
            const scope = {
                params,
                strategy,
                timeout,
                feedback,
                push,
                unpush,
                cancelAll,
                inFlight,
            };
            const go = run(scope, getHandler(), fin(scope, anyway, 0, req.rs), fin(scope, anyway, 1, req.rj), fin(scope, cancelled, 2, req.rj));
            if (running.length >= limit) {
                return go(new LimitExceededError(limit, running.length));
            }
            if (strategy === TAKE_FIRST && running.length > 0) {
                return go(new CancelledError(strategy));
            }
            if (strategy === TAKE_LAST && running.length > 0) {
                cancelAll(strategy);
            }
            if (strategy === QUEUE && running.length > 0) {
                const promise = cancellable(Promise.all(running.map(p => p.catch(() => { }))));
                push(promise);
                return promise.then(() => (unpush(promise), go()), error => (unpush(promise), go(error)));
            }
            go();
        },
    }),
];
const run = ({ params, push, timeout }, handler, onResolve, onReject, onCancel) => (immediatelyCancelError) => {
    if (immediatelyCancelError) {
        return onCancel()(immediatelyCancelError);
    }
    let cancel;
    let result;
    try {
        result = handler(params, abort => (cancel = abort));
    }
    catch (error) {
        return onReject()(error);
    }
    if (Object(result) === result && typeof result.then === 'function') {
        const promise = cancellable(result, cancel, timeout);
        push(promise);
        return promise.then(onResolve(promise), error => error instanceof CancelledError
            ? onCancel(promise)(error)
            : onReject(promise)(error));
    }
    return onResolve()(result);
};
const fin = ({ params, unpush, strategy, feedback, cancelAll, inFlight }, target, type, fn) => (promise) => (data) => {
    const runningCount = unpush(promise);
    const targets = [inFlight, sidechain];
    const payloads = [runningCount, [fn, data]];
    if (type === 2 || !runningCount || strategy === RACE) {
        const body = type === 0
            ? { params, result: data, status: 'done' }
            : type === 1
                ? { params, error: data, status: 'fail' }
                : { params, error: data };
        if (feedback)
            body.strategy = strategy;
        targets.unshift(target);
        payloads.unshift(body);
        if (strategy === RACE && type !== 2) {
            cancelAll(strategy);
        }
    }
    launch({
        target: targets,
        params: payloads,
        defer: true,
    });
};
const sidechain = createNode({
    node: [
        step.run({
            fn([fn, value]) {
                fn(value);
            },
        }),
    ],
    meta: { op: 'fx', fx: 'sidechain' },
});
