import { Effect, Event, Step } from 'effector';
import { ReEffectError } from './error';
import { Strategy } from './strategy';
export interface CreateReEffectConfig<Payload, Done> {
    handler?: Handler<Payload, Done>;
    name?: string;
    sid?: string;
    strategy?: Strategy;
    feedback?: boolean;
    limit?: number;
    timeout?: number;
}
export interface ReEffectConfig<Payload> {
    params?: Payload;
    strategy?: Strategy;
    timeout?: number;
}
interface CallableReEffect<Payload, Done> {
    (params: Payload, strategy?: Strategy): Promise<Done>;
    (params: Payload, config?: ReEffectConfig<Payload>): Promise<Done>;
    (config?: ReEffectConfig<Payload>): Promise<Done>;
}
export interface ReEffect<Payload, Done, Fail = Error> extends CallableReEffect<Payload, Done>, Omit<Effect<Payload, Done, Fail>, 'done' | 'fail' | 'finally' | 'use'> {
    readonly done: Event<DonePayload<Payload, Done>>;
    readonly fail: Event<FailPayload<Payload, Fail>>;
    readonly finally: Event<FinallyPayload<Payload, Done, Fail>>;
    readonly cancelled: Event<CancelledPayload<Payload>>;
    readonly cancel: Event<void>;
    readonly use: {
        (handler: Handler<Payload, Done>): ReEffect<Payload, Done, Fail>;
        getCurrent(): Handler<Payload, Done>;
    };
}
export interface MutableReEffect<Payload, Done, Fail = Error> extends CallableReEffect<Payload, Done>, Mutable<ReEffect<Payload, Done, Fail>> {
    graphite: Step;
    create: (paramsOrConfig: Payload | ReEffectConfig<Payload> | undefined, _: any, [maybeStrategyOrConfig]: [ReEffectConfig<Payload> | Strategy | undefined]) => Promise<Done>;
}
export declare type Handler<Payload, Done> = (payload: Payload, onCancel: (callback: () => any) => void) => Promise<Done> | Done;
export interface CreateReEffect {
    <Payload, Done, Fail = Error>(): ReEffect<Payload, Done, Fail>;
    <Payload, Done, Fail = Error>(name: string): ReEffect<Payload, Done, Fail>;
    <Payload, Done, Fail = Error>(config: CreateReEffectConfig<Payload, Done>): ReEffect<Payload, Done, Fail>;
    <Payload, Done, Fail = Error>(name: string, config: CreateReEffectConfig<Payload, Done>): ReEffect<Payload, Done, Fail>;
}
export declare type DonePayload<Payload, Done> = {
    params: Payload;
    strategy?: Strategy;
    result: Done;
};
export declare type FailPayload<Payload, Fail> = {
    params: Payload;
    strategy?: Strategy;
    error: Fail;
};
export declare type FinallyPayload<Payload, Done, Fail> = (DonePayload<Payload, Done> & {
    status: 'done';
}) | (FailPayload<Payload, Fail> & {
    status: 'fail';
});
export declare type CancelledPayload<Payload> = {
    params: Payload;
    strategy?: Strategy;
    error: ReEffectError;
};
declare type Mutable<T> = {
    -readonly [P in keyof T]: T[P];
};
export {};
